
from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.full_type_declaration.identifier)
lAlign.append(token.incomplete_type_declaration.identifier)
lAlign.append(token.file_declaration.identifier)
lAlign.append(token.signal_declaration.identifier)
lAlign.append(token.constant_declaration.identifier)
lAlign.append(token.subtype_declaration.identifier)
lAlign.append(token.variable_declaration.identifier)
lAlign.append(token.alias_declaration.alias_designator)

lUnless = []
lUnless.append([token.subprogram_body.is_keyword,token.subprogram_body.begin_keyword])


class rule_029(align_tokens_in_region_between_tokens_unless_between_tokens):
    '''
    This rule checks for alignment of names in alias, attribute, type, subtype, constant, signal, variable and file declarations in the architecture declarative region.

    |configuring_identifier_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal    sig1 : std_logic;
       file some_file :
       variable v_var1 : std_logic;
       type t_myType : std_logic;

    **Fix**

    .. code-block:: vhdl

       signal   sig1 : std_logic;
       file     some_file :
       variable v_var1 : std_logic;
       type     t_myType : std_logic;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_unless_between_tokens.__init__(self, 'architecture', '029', lAlign, token.architecture_body.is_keyword, token.architecture_body.begin_keyword, lUnless)
        self.solution = 'Align identifer.'




from vsg.rules import insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token

from vsg.token import context_declaration as token


class rule_022(insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token):
    '''
    This rule checks for the context name in the **end context** statement.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end context;

    **Fix**

    .. code-block:: vhdl

       end context c1;
    '''

    def __init__(self):
        insert_token_left_of_token_if_it_does_not_exist_between_tokens_using_value_from_token.__init__(self, 'context', '022', token.context_simple_name, token.semicolon, token.end_keyword, token.semicolon, token.identifier)
        self.solution = 'context simple same'
        self.groups.append('structure::optional')




from .rule_001 import rule_001
from .rule_002 import rule_002
from .rule_003 import rule_003
from .rule_004 import rule_004
from .rule_005 import rule_005
from .rule_006 import rule_006
from .rule_007 import rule_007

from .rule_010 import rule_010
from .rule_011 import rule_011
from .rule_012 import rule_012

from .rule_017 import rule_017

from .rule_400 import rule_400

from .rule_600 import rule_600




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.report_statement.severity_keyword)


class rule_501(token_case):
    '''
    This rule checks the **severity** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

        report "FIFO width is limited to 16 bits."
          SEVERITY FAILURE;

    .. code-block:: vhdl

        report "FIFO width is limited to 16 bits."
          severity FAILURE;
    '''

    def __init__(self):
        token_case.__init__(self, 'report_statement', '501', lTokens)
        self.groups.append('case::keyword')




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.component_instantiation_statement.instantiation_label)


class rule_008(token_case_with_prefix_suffix):
    '''
    This rule checks the instance label has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       U_FIFO : fifo

    **Fix**

    .. code-block:: vhdl

       u_fifo : fifo
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'instantiation', '008', lTokens)
        self.groups.append('case::label')




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.procedure_specification.procedure_keyword)


class rule_500(token_case):
    '''
    This rule checks the **procedure** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       PROCEDURE average_samples is

    **Fix**

    .. code-block:: vhdl

       procedure average_samples is
    '''

    def __init__(self):
        token_case.__init__(self, 'procedure', '500', lTokens)
        self.groups.append('case::keyword')




from vsg import parser


class generate_statement_label(parser.label):
    '''
    unique_id = block_specification : generate_statement_label
    '''

    def __init__(self, sString):
        parser.label.__init__(self, sString)


class open_parenthesis(parser.open_parenthesis):
    '''
    unique_id = block_specification : open_parenthesis
    '''

    def __init__(self, sString='('):
        parser.open_parenthesis.__init__(self)


class close_parenthesis(parser.close_parenthesis):
    '''
    unique_id = block_specification : close_parenthesis
    '''

    def __init__(self, sString=')'):
        parser.close_parenthesis.__init__(self)


class architecture_name(parser.name):
    '''
    unique_id = block_specification : architecture_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.block_statement.end_keyword)


class rule_302(token_indent):
    '''
    This rule checks the indent of the **end** keyword.

    **Violation**

    .. code-block:: vhdl

       block_label : block is

       begin

         end block block_label;

    **Fix**

    .. code-block:: vhdl

       block_label : block is

       begin

       end block block_label;
    '''

    def __init__(self):
        token_indent.__init__(self, 'block', '302', lTokens)




from vsg import deprecated_rule


class rule_008(deprecated_rule.Rule):
    '''
    The function of this rule has been included in rule case_201.
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'case', '008')
        self.message.append('The function of rule ' + self.unique_id + ' has been included in rule case_201:')
